unit uOpcionesSimSEEEdit;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFDEF WINDOWS}
  Windows,
{$ENDIF}
{$IFDEF FPC}
  FileUtil,
{$ENDIF}
  uCosa, uconstantesSimSEE, SysUtils, Dialogs, utilidades;

resourcestring
  mesErrorLeyendoArchivoPref =
    'Ocurrio un error leyendo el archivo de preferencias del editor.';
  mesUsaronValoresXDefecto = 'Se usaran los valores por defecto';

const
  configFile = '.\config.ini';

type
  TSimSEEEditOptions = class(TCosa)
  private
    fLibPath: string;

    constructor Create(capa: integer; fechasAutomaticas: boolean;
      deshabilitarScrollHorizontalEnListados: boolean; libPath: string;
      guardarBackupDeArchivos: boolean; maxNBackups: integer);
  public
    fechasAutomaticas: boolean;
    deshabilitarScrollHorizontalEnListados: boolean;
    guardarBackupDeArchivos: boolean;
    maxNBackups: integer;

    //Van public por el warning. No usar, usar getInstance
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;

    class function getInstance: TSimSEEEditOptions;
    procedure setLibPath(newLibPath: string);
    procedure guardar;
    procedure Free; override;

    property libPath: string read fLibPath write setLibPath;
  end;

implementation

var
  opciones: TSimSEEEditOptions;

constructor TSimSEEEditOptions.Create(capa: integer; fechasAutomaticas: boolean;
  deshabilitarScrollHorizontalEnListados: boolean; libPath: string;
  guardarBackupDeArchivos: boolean; maxNBackups: integer);
begin
  if opciones <> nil then
    raise Exception.Create(
      'TSimSEEEditOptions.Create: Error se esta creando una segunda instancia de TSimSEEEditOptions');

  inherited Create(capa);
  self.fechasAutomaticas := fechasAutomaticas;
  self.deshabilitarScrollHorizontalEnListados := deshabilitarScrollHorizontalEnListados;
  self.libPath := libPath;
  setLibPath(libPath);
  opciones := self;
  self.guardarBackupDeArchivos := guardarBackupDeArchivos;
  Self.maxNBackups := maxNBackups;
end;

constructor TSimSEEEditOptions.Create_ReadFromText(f: TArchiTexto);
begin
  if opciones <> nil then
    raise Exception.Create(
      'TSimSEEEditOptions.Create_ReadFromText: Error se esta creando una segunda instancia de TSimSEEEditOptions');

  if f.Version < 14 then
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('fechasAutomaticas', fechasAutomaticas);
    f.rd('libPath', fLibPath);
    f.EjecutarLectura;
    setLibPath(libPath);

    deshabilitarScrollHorizontalEnListados := True;

    guardarBackupDeArchivos := True;
    maxNBackups := 10;
  end
  else if f.Version < 15 then
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('fechasAutomaticas', fechasAutomaticas);
    f.rd('deshabilitarScrollHorizontalEnListados',
      deshabilitarScrollHorizontalEnListados);
    f.rd('libPath', fLibPath);
    f.EjecutarLectura;
    setLibPath(libPath);

    guardarBackupDeArchivos := True;
    maxNBackups := 10;
  end
  else
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('fechasAutomaticas', fechasAutomaticas);
    f.rd('deshabilitarScrollHorizontalEnListados',
      deshabilitarScrollHorizontalEnListados);
    f.rd('libPath', fLibPath);
    setLibPath(libPath);
    f.rd('guardarBackupDeArchivos', guardarBackupDeArchivos);
    f.rd('maxNBackups', maxNBackups);
    f.EjecutarLectura;
  end;
  opciones := self;
end;

procedure TSimSEEEditOptions.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('fechasAutomaticas', fechasAutomaticas);
  f.wr('deshabilitarScrollHorizontalEnListados', deshabilitarScrollHorizontalEnListados);
  f.wr('libPath', quitarRaiz_(fLibPath));
  f.wr('guardarBackupDeArchivos', guardarBackupDeArchivos);
  f.wr('maxNBackups', maxNBackups);
end;

class function TSimSEEEditOptions.getInstance: TSimSEEEditOptions;
var
  f: TArchiTexto;
begin
  if opciones = nil then
  begin
    if FileExists(configFile) { *Converted from FileExists*  } then
    begin
      f := nil;
      try
        f := TArchiTexto.CreateForRead(getDir_Bin + configFile, True);
        f.rd('opciones', TCosa(opciones));
        f.Free;
      except
        ShowMessage(mesErrorLeyendoArchivoPref + mesUsaronValoresXDefecto);
        if f <> nil then
          f.Free;
        DeleteFile(configFile);
        opciones := TSimSEEEditOptions.Create(0, True, True,
          uconstantesSimSEE.getDir_Lib, True, 10);
      end;
    end
    else
    begin
      opciones := TSimSEEEditOptions.Create(0, True, False,
        uconstantesSimSEE.getDir_Lib, True, 10);
    end;
  end;

  Result := opciones;
end;

procedure TSimSEEEditOptions.setLibPath(newLibPath: string);
begin
  //Si no es una url de red le pongo la letra de unidad
  if (pos('\\', newLibPath) <> 1) and (pos(':\', newLibPath) = 0) then
    fLibPath := ExtractFileDrive(getDir_Bin) + newLibPath
  else
    fLibPath := newLibPath;
  if fLibPath[Length(fLibPath)] <> barraDir then
    fLibPath := fLibPath + barraDir;
  if not DirectoryExists(fLibPath) { *Converted from DirectoryExists*  } then
    if not ForceDirectories(fLibPath) { *Converted from ForceDirectories*  } then
      raise Exception.Create(
        'TSimSEEEditOptions.setLibPath: No se puede acceder al directorio ' +
        newLibPath);
end;

procedure TSimSEEEditOptions.guardar;
var
  f: TArchiTexto;
begin
  f := nil;
  try
    ucosa.procMsgValorPorDefecto := nil;
    f := TArchiTexto.CreateForWrite(getDir_Bin + configFile, False, 0);
    f.wr('opciones', opciones);
  finally
    if f <> nil then
      f.Free;
  end;
end;

procedure TSimSEEEditOptions.Free;
begin
  opciones := nil;
  self.Free;
end;

end.
