{+doc
+NOMBRE:   uParqueEolico
+CREACION: 22/12/2006
+AUTORES:  rch
+REGISTRO:
+TIPO: Unidad Pascal.
+PROPOSITO:   Definición del actor TParqueEolico
+PROYECTO: SimSEE
+REVISION:
+AUTOR:
+DESCRIPCION:

-doc}

unit uParqueEolico;


interface

uses
  SysUtils,
  Classes,
  uFechas,
  uGeneradores,
  uFuentesAleatorias,
  uGlobs,
  uNodos,
  uCosa,
  usimplex,
  uFichasLPD,
  uFuncionesReales,
  xMatDefs;

resourcestring
  mesFuenteAleatConPaso1h =
    'TParqueEolico, necesita una fuente aleatoria con paso de sorteo = 1h. ';
  mesElParque = 'El parque ';
  mesConectadoAFuente = ' está conectado a la fuente ';
  mesConPasoDeSorteo = ' con paso de sorteo = ';
  rsParqueEolico = 'Parque eólico';
  exCantSpeedupParque = 'La cantidad de factores de speedup en el parque eolico ';
  exDebeSer12 = ' debe ser 12';

type

  TParqueEolico = class(TGenerador)
  public
    PagoPorEnergiaDisponible_USD_MWh: NReal;    //[USD/MWh] Pago por potencia disponible contemplando las unidades disponibles
    // resultado de los sorteos de disponibilidad y del mantenimiento programado y el viento
    PagoPorEnergiaEntregada_USD_MWh: NReal; // [USD/MWh] para cálculo de CAD

    // fuente de viento
    fuenteDeViento: TFuenteAleatoria;
    nombreBorne: string;
    kBorneVelocidad: integer;
    kBornePPA: TDAofNInt; // Potencia del Poste Actual

    // parámetros editables
    fdisp: NReal;
    tRepHoras: NReal;
    fSpeedUpMes: TDAofNReal;
    fPerdidasInterferencias: NReal;
    fComplexivoMes: TDAofNReal;
    CurvaVP: TFVectR;

    NMaquinasDisponibles: integer;
    VVel: NReal;
    PPA: TDAOfNReal;


    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; nodo: TNodo; xFDV: TFuenteAleatoria;
  xNombreBorne: string; xfdisp: NReal; tRepHoras: NReal;
  xfSpeedUpMes: TDAofNReal; xfPerdidasInterferencias: NReal;
  xCurvaVelocidadPotencia_Pot: TFVectR;
  xPagoPorEnergiaEntregada_USD_MWh: NReal;
  xPagoPorEnergiaDisponible_USD_MWh: NReal; TonCO2xMWh: NReal; LowCostMustRun,
  CleanDevelopmentMechanism: boolean);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure PrepararMemoria(globs: TGlobs); override;

    function PotenciaFirme: NReal; override;

    function InfoAd_: string; override;

    class function DescClase: string; override;

    function get_pa_FD( kTipoUnidad: integer ): NReal; override;
    function get_pa_TMR( kTipoUnidad: integer ): NReal; override;

    procedure SorteosDelPaso(sortear: boolean); override;
    procedure PrepararPaso_ps; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;

    procedure opt_cargue(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_leerSolucion(s: TSimplex); override;
    {$IFDEF SPXCONLOG}
    procedure spx_NombrarVariables(s: TSimplex); override;
    {$ENDIF}
    procedure Free; override;
    procedure PubliVars; override;

    procedure dump_Variables(var f: TextFile; charIndentacion: char); override;

    function getNombreVar(ivar: integer; var nombre: string): boolean; override;

  end;

procedure AlInicio;
procedure AlFinal;

implementation


constructor TParqueEolico.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades: TFichasLPD; nodo: TNodo; xFDV: TFuenteAleatoria;
  xNombreBorne: string; xfdisp: NReal; tRepHoras: NReal; xfSpeedUpMes: TDAofNReal;
  xfPerdidasInterferencias: NReal; xCurvaVelocidadPotencia_Pot: TFVectR;
  xPagoPorEnergiaEntregada_USD_MWh: NReal; xPagoPorEnergiaDisponible_USD_MWh: NReal; TonCO2xMWh: NReal;
  LowCostMustRun, CleanDevelopmentMechanism: boolean);
var
  i: integer;
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, nodo,
    TonCO2xMWh, LowCostMustRun, CleanDevelopmentMechanism);

  fdisp := xfdisp;
  self.tRepHoras := tRepHoras;
  if length(xfSpeedUpMes) <> 12 then
    raise Exception.Create(exCantSpeedupParque + nombre + exDebeSer12);
  fSpeedUpMes := xfSpeedUpMes;
  fPerdidasInterferencias := xfPerdidasInterferencias;
  SetLength(fComplexivoMes, 12);
  for i := 0 to high(fSpeedUpMes) do
    fComplexivoMes[i] := fSpeedUpMes[i] * fPerdidasInterferencias;

  CurvaVP := xCurvaVelocidadPotencia_Pot;
  FuenteDeViento := xFDV;
  NombreBorne := xNombreBorne;
  kBorneVelocidad := -1;
  PagoPorEnergiaEntregada_USD_MWh := xPagoPorEnergiaEntregada_USD_MWh;
  PagoPorEnergiaDisponible_USD_MWh := xPagoPorEnergiaDisponible_USD_MWh;
end;

procedure TParqueEolico.Free;
begin
  CurvaVP.Free;

  SetLength(kBornePPA, 0);
  SetLength(fSpeedUpMes, 0);
  SetLength(fComplexivoMes, 0);
  SetLength(PPA, 0);

  inherited Free;
end;

procedure TParqueEolico.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('PotenciaGenerable', '[MW]', 6, 2, PPA, True, True);
  PublicarVariableNR('VVel', '[m/s]', 6, 2, VVel, True);
  PublicarVariableNI('NMaqsDisponibles', '-', NMaquinasDisponibles, True);
  PublicarVariableNR('CostoDirectoDelPaso', '[USD]', 12, 2,
    self.costoDirectoDelPaso, True);
end;

constructor TParqueEolico.Create_ReadFromText(f: TArchiTexto);
var
  i: integer;
  fSpeedUp: NReal;
begin
  PagoPorEnergiaDisponible_USD_MWh:= 0;
  PagoPorEnergiaEntregada_USD_MWh:= 0;

  if f.Version < 7 then
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('fdisp', fdisp);
    f.rd('fSpeedUp', fSpeedUp);
    f.rd('fPerdidasInterferencias', fPerdidasInterferencias);
    f.rd('CurvaVP', TCosa(CurvaVP));
    f.rdReferencia('fuenteDeViento', TCosa(fuenteDeViento), Self);
    f.rd('nombreBorne', nombreBorne);
    f.EjecutarLectura;

    SetLength(fSpeedUpMes, 12);
    SetLength(fComplexivoMes, 12);
    for i := 0 to high(fSpeedUpMes) do
    begin
      fSpeedUpMes[i] := fSpeedUp;
      fComplexivoMes[i] := fPerdidasInterferencias * fSpeedUP;
    end;
    kBorneVelocidad := -1;

    self.tRepHoras := 15 * 24;
  end
  else if f.Version < 14 then
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('fdisp', fdisp);
    f.rd('fSpeedUpMes', fSpeedUPMes);
    f.rd('fPerdidasInterferencias', fPerdidasInterferencias);
    f.rd('CurvaVP', TCosa(CurvaVP));
    f.rdReferencia('fuenteDeViento', TCosa(fuenteDeViento), Self);
    f.rd('nombreBorne', nombreBorne);
    f.EjecutarLectura;

    SetLength(fComplexivoMes, 12);
    for i := 0 to high(fSpeedUpMes) do
      fComplexivoMes[i] := fPerdidasInterferencias * fSpeedUPMes[i];
    kBorneVelocidad := -1;

    self.tRepHoras := 15 * 24;
  end
  else
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('fdisp', fdisp);
    f.rd('tRepHoras', tRepHoras);
    f.rd('fSpeedUpMes', fSpeedUPMes);
    f.rd('fPerdidasInterferencias', fPerdidasInterferencias);
    f.rd('CurvaVP', TCosa(CurvaVP));
    f.rdReferencia('fuenteDeViento', TCosa(fuenteDeViento), Self);
    f.rd('nombreBorne', nombreBorne);
    f.EjecutarLectura;

    SetLength(fComplexivoMes, 12);
    for i := 0 to high(fSpeedUpMes) do
      fComplexivoMes[i] := fPerdidasInterferencias * fSpeedUPMes[i];
    kBorneVelocidad := -1;

    if f.version < 98 then
    begin
      if f.Version >= 36 then
        f.rd('pagoPorEnergia', PagoPorEnergiaEntregada_USD_MWh);
      f.rd('pagoPorPotenciaDisp', PagoPorEnergiaDisponible_USD_MWh );
    end
    else if f.version < 103 then
    begin
      f.rd('PagoPorDisponibilidad_USD_MWh', PagoPorEnergiaDisponible_USD_MWh );
      f.rd('PagoPorEnergia_USD_MWh', PagoPorEnergiaEntregada_USD_MWh)
    end
    else
    begin
      f.rd('PagoPorEnergiaDisponible_USD_MWh', PagoPorEnergiaDisponible_USD_MWh );
      f.rd('PagoPorEnergiaEntregada_USD_MWh', PagoPorEnergiaEntregada_USD_MWh)
    end
  end;
end;

procedure TParqueEolico.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('fdisp', fdisp);
  f.wr('tRepHoras', tRepHoras);
  f.wr('fSpeedUpMes', fSpeedUpMes);
  f.wr('fPerdidasInterferencias', fPerdidasInterferencias);
  f.wr('CurvaVP', CurvaVP);
  f.wrReferencia('fuenteDeViento', fuenteDeViento);
  f.wr('nombreBorne', nombreBorne);
  f.wr('PagoPorEnergiaDisponible_USD_MWh', PagoPorEnergiaDisponible_USD_MWh );
  f.wr('PagoPorEnergiaEntregada_USD_MWh', PagoPorEnergiaEntregada_USD_MWh)

end;

procedure TParqueEolico.prepararMemoria(globs: TGlobs);
var
  iposte: integer;
  auxf: TFf_xmult_conselector;
begin
  inherited prepararMemoria(globs);
  setlength(PPA, globs.NPostes);

  if not ((fuenteDeViento.durPasoDeSorteoEnHoras = 1) or
    ((fuenteDeViento.durPasoDeSorteoEnHoras = 0) and (globs.HorasDelPaso = 1))) then
    raise Exception.Create(mesFuenteAleatConPaso1h + mesElParque +
      nombre + mesConectadoAFuente + fuenteDeViento.nombre +
      mesConPasoDeSorteo + IntToStr(fuenteDeViento.durPasoDeSorteoEnHoras));

  if kBorneVelocidad = -1 then // la primera vez definimos los bornes
  begin
    kBorneVelocidad := fuenteDeViento.idBorne(nombreBorne);
    SetLength(kBornePPA, globs.NPostes);
    for iposte := 0 to high(kBornePPA) do
    begin
       (*
       Para cada poste definimos un borne auxiliar que llevará la cuenta
       de la energía generada por el parque en ese poste y así puede calcular
       la potencia media equivalente del parque para ese poste.
       Se supone que la fuente de viento tiene paso de sorteo HORARIO.
       *)

      auxf := TFf_xmult_conselector.Create(capa, CurvaVP, fComplexivoMes,
        // speedUp * fpInterferencias
        1, @globs.MesInicioDelPaso, iposte, @globs.kPosteHorasDelPaso,
        @globs.kSubPaso_);
      kBornePPA[iposte] := fuenteDeViento.registrarFuncion(auxf, nombreBorne, iposte);
    end;
  end;
end;

function TParqueEolico.potenciaFirme: NReal;
begin
  Result := 0;
end;

function TParqueEolico.infoAd_: string;
begin
  Result := '';
end;

class function TParqueEolico.DescClase: string;
begin
  Result := rsParqueEolico;
end;


function TParqueEolico.get_pa_FD( kTipoUnidad: integer ): NReal;
begin
  result:= fdisp;
end;

function TParqueEolico.get_pa_TMR( kTipoUnidad: integer ): NReal;
begin
  result:= tRepHoras;
end;



procedure TParqueEolico.SorteosDelPaso(sortear: boolean);
begin
  if globs.ObligarDisponibilidad_1_ then
    NMaquinasDisponibles := paUnidades.nUnidades[0]
  else if (sortear) then
  begin
    ActualizarProbabilidadesReparacionYRotura_(fdisp, tRepHoras);
    NMaquinasDisponibles := Sorteos_RepRotUnidades;
  end
  else
    NMaquinasDisponibles := paUnidades.nUnidades[0];
end;

procedure TParqueEolico.PrepararPaso_ps;
var
  iposte: integer;
begin
  VVel := Self.fuenteDeViento.bornera[Self.kBorneVelocidad] *
    fSpeedUpMes[globs.MesInicioDelPaso - 1];
  for iposte := 0 to high(PPA) do
    PPA[iposte] := Self.fuenteDeViento.bornera[Self.kBornePPA[iposte]] *
      NMaquinasDisponibles;
end;

procedure TParqueEolico.opt_nvers(var ivar, ivae, ires: integer);
begin
  self.ivar := ivar;
  self.ivae := ivae;
  self.ires := ires;
  ivar := ivar + globs.NPostes;
end;

function TParqueEolico.getNombreVar(ivar: integer; var nombre: string): boolean;
begin
  if (ivar >= self.ivar) and (ivar < self.ivar + globs.NPostes) then
  begin
    nombre := self.Nombre + '_P[MW]' + IntToStr(ivar - self.ivar + 1);
    Result := True;
  end
  else
    Result := False;
end;


procedure TParqueEolico.opt_cargue(s: TSimplex);
var
  iposte: integer;
begin
  if NMaquinasDisponibles = 0 then
    exit;
{$IFDEF SPXCONLOG}
  spx_NombrarVariables(s);
{$ENDIF}
  for iposte := 0 to globs.NPostes - 1 do
    s.pon_e(nodo.ires + iposte, ivar + iposte, 1);
end;

procedure TParqueEolico.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  iposte: integer;
begin
  if NMaquinasDisponibles = 0 then
    exit;
  // Le fijamos como cota superior PMax a la potencia en todos los postes
  for iposte := 0 to globs.NPostes - 1 do
    if PPA[iposte] > AsumaCero then
      s.cota_sup_set(ivar + iposte, PPA[iposte])
    else
      s.FijarVariable(ivar + iposte, 0);
end;

procedure TParqueEolico.opt_leerSolucion(s: TSimplex);
var
  iposte: integer;
  e, eacum, disp: NReal;
begin
  // recuperamos los valores de Potencia despachada
  eacum := 0;
  disp := 0;
  for iposte := 0 to globs.NPostes - 1 do
  begin
    e := s.xval(ivar + iposte);
    P[iposte] := e;
    eacum := eacum + e * globs.DurPos[iposte];
    disp := disp + PPA[iposte] * globs.DurPos[iposte];
  end;

  Ingreso_PorDisponibilidad:=  disp * PagoPorEnergiaDisponible_USD_MWh;
  Ingreso_PorEnergia:= eacum * PagoPorEnergiaEntregada_USD_MWh;

  costoDirectoDelPaso := Ingreso_PorEnergia + Ingreso_PorDisponibilidad;
end;

{$IFDEF SPXCONLOG}
procedure TParqueEolico.spx_NombrarVariables(s: TSimplex);
var
  iposte: integer;
begin
  for iposte := 0 to globs.NPostes - 1 do
    s.set_NombreVar(ivar + iposte, Nombre + '_P[MW]' + IntToStr(iposte + 1));
end;

{$ENDIF}

procedure TParqueEolico.dump_Variables(var f: TextFile; charIndentacion: char);
var
  i: integer;
begin
  inherited dump_Variables(f, charIndentacion);
  writeln(f, charIndentacion, 'VVel[m/s]= ', FloatToStrF(VVel, ffFixed, 10, 3));
  for i := 0 to high(PPA) do
    writeln(f, charIndentacion, 'PPA[', i, '][MW]: ',
      FloatToStrF(PPA[i], ffFixed, 10, 3));
  writeln(f);
  //  writeln(f, charIndentacion, 'PotMediaGenerable[MW]= ', FloatToStrF(PotMediaGenerable, ffFixed, 10, 3));
  writeln(f, charIndentacion, 'NMaquinasDisponibles= ', NMaquinasDisponibles);
  writeln(f);
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TParqueEolico.ClassName, TParqueEolico);
end;

procedure AlFinal;
begin
end;

end.
